@@ -1,4 +1,5 @@
 import os
+import socket
 import tempfile
 import time
 import youtube_dl 
@@ -13,10 +14,25 @@ from musicazoo.lib.watch_dl import WatchCartoonOnlineIE
 from youtube_dl.compat import compat_cookiejar, compat_urllib_request
 from youtube_dl.utils import make_HTTPS_handler, YoutubeDLHandler
 
+import threading
+import urllib2
+
 messages = Queue.Queue()
 
+def get_mime_type(url):
+    class HeadRequest(urllib2.Request):
+        def get_method(self):
+            return "HEAD"
+    try:
+        response = urllib2.urlopen(HeadRequest(url))
+        return response.info().dict['content-type']
+    except Exception as e:
+        raise Exception("URL Error")
+
 class YoutubeModule(pymodule.JSONParentPoller):
-    def __init__(self):
+    def __init__(self, headless=False):
+        self.headless=headless
+        self.update_lock = threading.Lock() # TODO I don't think this needs to exist
         self.thread_stopped = False 
         super(YoutubeModule, self).__init__()
 
@@ -47,8 +63,6 @@ class YoutubeModule(pymodule.JSONParentPoller):
         return self.state_has_started and not (self.state_is_suspended or self.state_is_paused)
 
     def cmd_init(self, url):
-        print "URL:", url
-
         # Has the youtube url/data been fetched?
         self.state_is_ready = False 
         # Has the video started playing at all?
@@ -72,10 +86,9 @@ class YoutubeModule(pymodule.JSONParentPoller):
         self.cookies=None
         self.rate=None
         messages.put("init")
-        self.update(self.serialize()) # TODO fix this
+        self.safe_update()
 
     def cmd_play(self):
-        print "Play"
         if self.state_has_started:
             if not self.state_is_paused:
                 self.vlc_mp.play()
@@ -83,34 +96,30 @@ class YoutubeModule(pymodule.JSONParentPoller):
         else:
             messages.put("play")
         self.state_is_suspended = False
-        self.update(self.serialize()) # TODO fix this
+        self.safe_update()
 
     def cmd_suspend(self):
-        print "Suspend"
         if self.state_has_started:
             if self.vlc_mp.is_playing():
                 self.vlc_mp.pause()
             #self.state_is_paused = True
         self.state_is_suspended = True
-        self.update(self.serialize()) # TODO fix this
+        self.safe_update()
 
     def cmd_resume(self):
-        print "Resume"
         if self.state_has_started:
             self.vlc_mp.play()
             self.state_is_paused = False
-            self.update(self.serialize()) # TODO fix this
+            self.safe_update()
 
     def cmd_pause(self):
-        print "Pause"
         if self.state_has_started:
             if self.vlc_mp.is_playing():
                 self.vlc_mp.pause()
             self.state_is_paused = True
-            self.update(self.serialize()) # TODO fix this
+            self.safe_update()
 
     def cmd_rm(self):
-        print "Remove"
         self.state_is_stopping = True
         messages.put("rm")
         #messages.join()
@@ -130,7 +139,9 @@ class YoutubeModule(pymodule.JSONParentPoller):
     def stop(self):
         self.vlc_mp.stop()
         self.thread_stopped = True
-        self.update_rm()
+
+        with self.update_lock:
+            self.rm()
 
     def play(self):
         def ev_end(ev):
@@ -138,14 +149,17 @@ class YoutubeModule(pymodule.JSONParentPoller):
 
         def ev_time(ev):
             self.time = ev.u.new_time / 1000.
-            self.update(self.serialize()) # TODO fix this
+            self.safe_update()
 
         def ev_length(ev):
             self.duration = ev.u.new_length / 1000.
-            self.update(self.serialize()) # TODO fix this
+            self.safe_update()
 
-        os.environ["DISPLAY"] = ":0"
-        self.vlc_i = vlc.Instance(['-f','--no-video-title-show','--no-xlib'])
+        if not self.headless:
+            os.environ["DISPLAY"] = ":0"
+            self.vlc_i = vlc.Instance(['-f','--no-video-title-show','--no-xlib'])
+        else:
+            self.vlc_i = vlc.Instance(['--novideo'])
         self.vlc_mp = self.vlc_i.media_player_new()
         self.vlc_ev = self.vlc_mp.event_manager()
 
@@ -155,66 +169,79 @@ class YoutubeModule(pymodule.JSONParentPoller):
 
         vlc_media=self.vlc_i.media_new_location(self.media)
         self.vlc_mp.set_media(vlc_media)
-        self.vlc_mp.set_xwindow(0)
-        self.vlc_mp.set_fullscreen(True)
+        if not self.headless:
+            self.vlc_mp.set_xwindow(0)
+            self.vlc_mp.set_fullscreen(True)
         self.vlc_mp.play()
 
         self.state_has_started = True
-        self.update(self.serialize()) # TODO fix this
+        self.safe_update()
 
     def get_video_info(self):
         url = self.url
-        # General configuration
-        tf=tempfile.NamedTemporaryFile(delete=False)
-        tf.close()
-        self.cookies=tf.name
-        jar = compat_cookiejar.MozillaCookieJar(self.cookies)
-        cookie_processor = compat_urllib_request.HTTPCookieProcessor(jar)
-        proxies = compat_urllib_request.getproxies()
-        proxy_handler = compat_urllib_request.ProxyHandler(proxies)
-        https_handler = make_HTTPS_handler(None)
-        opener = compat_urllib_request.build_opener(https_handler, proxy_handler, cookie_processor, YoutubeDLHandler)
-        compat_urllib_request.install_opener(opener)
+        mimetype=get_mime_type(url)
 
-        y=youtube_dl.YoutubeDL({'outtmpl':u'','skip_download':True}) # empty outtmpl needed due to weird issue in youtube-dl
-        y.add_info_extractor(WatchCartoonOnlineIE())
-        y.add_default_info_extractors()
+        if mimetype.startswith("text/html"):
+            # General configuration
+            tf=tempfile.NamedTemporaryFile(delete=False)
+            tf.close()
+            self.cookies=tf.name
+            jar = compat_cookiejar.MozillaCookieJar(self.cookies)
+            cookie_processor = compat_urllib_request.HTTPCookieProcessor(jar)
+            proxies = compat_urllib_request.getproxies()
+            proxy_handler = compat_urllib_request.ProxyHandler(proxies)
+            https_handler = make_HTTPS_handler(None)
+            opener = compat_urllib_request.build_opener(https_handler, proxy_handler, cookie_processor, YoutubeDLHandler)
+            compat_urllib_request.install_opener(opener)
 
-        try:
-            info=y.extract_info(url,download=False)
-        except Exception:
-            raise
-            self.status='invalid'
-            self.queue.removeMeAsync(self.uid) # Remove if possible
-            self.ready.release()
-            return False
+            y=youtube_dl.YoutubeDL({'outtmpl':u'','skip_download':True}, auto_init=False) # empty outtmpl needed due to weird issue in youtube-dl
+            y.add_info_extractor(WatchCartoonOnlineIE())
+            y.add_default_info_extractors()
 
-        jar.save()
+            try:
+                info=y.extract_info(url,download=False)
+            except Exception:
+                raise
+                self.status='invalid'
+                self.queue.removeMeAsync(self.uid) # Remove if possible # TODO error here
+                self.ready.release()
+                return False
+
+            jar.save()
+
+            if 'entries' in info:
+                vinfo=info['entries'][0]
+            else:
+                vinfo=info
+
+            if 'title' in vinfo:
+                self.title=vinfo['title']
+            if 'duration' in vinfo:
+                self.duration=vinfo['duration']
+            if 'extractor' in vinfo:
+                self.site=vinfo['extractor']
+            if 'url' in vinfo:
+                self.media=vinfo['url']
+            if 'thumbnail' in vinfo:
+                self.thumbnail=vinfo['thumbnail']
+            if 'description' in vinfo:
+                self.description=vinfo['description']
+            if 'id' in vinfo:
+                self.vid=vinfo['id']
 
-        if 'entries' in info:
-            vinfo=info['entries'][0]
         else:
-            vinfo=info
-
-        if 'title' in vinfo:
-            self.title=vinfo['title']
-        if 'duration' in vinfo:
-            self.duration=vinfo['duration']
-        if 'extractor' in vinfo:
-            self.site=vinfo['extractor']
-        if 'url' in vinfo:
-            self.media=vinfo['url']
-        if 'thumbnail' in vinfo:
-            self.thumbnail=vinfo['thumbnail']
-        if 'description' in vinfo:
-            self.description=vinfo['description']
-        if 'id' in vinfo:
-            self.vid=vinfo['id']
+            self.media=url
+            self.title=url
 
         self.state_is_ready = True
-        self.update(self.serialize()) # TODO fix this
+        self.safe_update()
         return True
 
+    # TODO This shouldn't exist.
+    def safe_update(self):
+        with self.update_lock:
+            self.set_parameters(self.serialize())
+
     commands={
         'init':cmd_init,
         'play':cmd_play,
@@ -227,10 +254,11 @@ class YoutubeModule(pymodule.JSONParentPoller):
         'do_seek_abs': cmd_seek_abs,
     }
 
-mod = YoutubeModule()
-
 import sys
-import threading
+
+headless = '--headless' in sys.argv
+
+mod = YoutubeModule(headless=headless)
 
 def serve_forever():
     while not mod.thread_stopped:
